#include <misc.h>
#include "dialog.h"
#include <proto.h>
#include "dialog.m"

class FIELD_BUTTON: public FIELD{
	SSTRING str;
	SSTRING icon;
	SSTRING help;	// Help text poping over the button
	/*~PROTOBEG~ FIELD_BUTTON */
public:
	FIELD_BUTTON (const char *_str,
		 const char *_icon,
		 const char *_help,
		 PRIVATE_MESSAGE&_msg);
	MENU_STATUS dokey (WINDOW *,
		 int key,
		 FIELD_MSG&,
		 bool&);
	void drawtxt (WINDOW *win, int);
	const char *get_registry_value (void);
	char getidprefix (void);
	void gui_draw (int nof, int &);
	void gui_get (int);
	void html_draw (int);
	int html_validate (int);
	void reload (const char *, int);
	void restore (void);
	void save (void);
	void set_registry_value (const char *);
	/*~PROTOEND~ FIELD_BUTTON */
};

PUBLIC FIELD_BUTTON::FIELD_BUTTON (
	const char *_str,
	const char *_icon,
	const char *_help,
	PRIVATE_MESSAGE &_msg)
	: FIELD (NULL)
{
	msg = &_msg;
	str.setfrom (_str);
	icon.setfrom (_icon);
	help.setfrom (_help);
}

PUBLIC MENU_STATUS FIELD_BUTTON::dokey(WINDOW *, int key, FIELD_MSG &, bool &)
{
	MENU_STATUS ret = MENU_NULL;
	if (key == ' '){
		dialog_sendmessage (*msg);
		ret = MENU_MESSAGE;
	}
	return ret;
}

PUBLIC void FIELD_BUTTON::set_registry_value(const char *)
{
}
PUBLIC const char *FIELD_BUTTON::get_registry_value()
{
	return NULL;
}


PUBLIC void FIELD_BUTTON::drawtxt(WINDOW *win, int)
{
	SSTRING s;
	if (!str.is_empty()){
		s.setfromf("[%s]",str.get());
	}else{
		// We present the icon name with the tag icon:
		s.setfromf("[%s:%s]",MSG_U(I_ICON,"Icon"),icon.get());
	}
	diaetc_drawinput (win,box.x,box.y,box.width,s.get());
}


PUBLIC void FIELD_BUTTON::save(){}
PUBLIC void FIELD_BUTTON::restore(){}
PUBLIC void FIELD_BUTTON::reload(const char *, int ){}
PUBLIC void FIELD_BUTTON::html_draw(int){}
PUBLIC void FIELD_BUTTON::gui_get(int ){}
PUBLIC char FIELD_BUTTON::getidprefix (){return ' '; }
PUBLIC int  FIELD_BUTTON::html_validate(int ){ return 0;}


PUBLIC void FIELD_BUTTON::gui_draw(int nof, int &)
{
	const char *ic = icon.get();
	if (ic[0] != '\0'){
		char sent[PATH_MAX];
		diagui_sendxpm (ic,sent);
		char tmp[1000];
		diagui_sendcmd (P_Button_xpm,"B%d %s $dump=1\n",200+nof
			,diagui_quote(sent,tmp));
	}else{
		char tmp[1000];
		diagui_sendcmd (P_Button,"B%d 1 %s\n",200+nof
			,diagui_quote(str.get(),tmp));
	}
}

/*
	Add a text button which will show up within the dialog area (wherever
	you want in fact, like any other field)

	See samples.cc: sample_extrabuttons()
*/
PUBLIC void DIALOG::new_button (
	const char *str,	// Text of the button
	const char *help,	// bubble test over the icon, optional
	PRIVATE_MESSAGE &msg)
{
	add (new FIELD_BUTTON (str,NULL,help,msg));
}

/*
	Add an icon button which will show up within the dialog area (wherever
	you want in fact, like any other field)

	See samples.cc: sample_extrabuttons()
*/
PUBLIC void DIALOG::new_button_icon (
	const char *icon,		// icon name, correponding to a xpm file
	const char *help,		// bubble test over the icon, optional
	PRIVATE_MESSAGE &msg)
{
	add (new FIELD_BUTTON (NULL,icon,help,msg));
}

