//
// This module was implemented by Gustavo Niemeyer <niemeyer@conectiva.com>
//

#pragma implementation
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <translat.h>
#include "kernelconf.h"
#include "kernelconf.m"

MODULE_DEFINE_VERSION(kernelconf);

PUBLIC MODULE_kernelconf::MODULE_kernelconf()
	: LINUXCONF_MODULE("kernelconf")
{
	linuxconf_loadmsg ("kernelconf",PACKAGE_REV);
}


static const char *keymenu=NULL;

PUBLIC void MODULE_kernelconf::setmenu (
	DIALOG &dia,
	MENU_CONTEXT context)
{
	if (context == MENU_MISCSERV){
		keymenu = MSG_U(M_kernelconf,"Kernel Configuration");
		dia.new_menuitem ("kernelconf","",keymenu);
	}
}

PUBLIC int MODULE_kernelconf::domenu (
	MENU_CONTEXT context,
	const char *key)
{
	if (context == MENU_MISCSERV){
		if (key == keymenu){
			kernelconf_edit();
		}
	}
	return 0;
}


PUBLIC int MODULE_kernelconf::dohtml (const char *key)
{
	int ret = LNCF_NOT_APPLICABLE;
	if (strcmp(key,"kernelconf")==0){
		// ### Insert any menu and dialog here
		ret = 0;
	}
	return ret;
}

PUBLIC int MODULE_kernelconf::probe (int level, int runlevel, bool sim)
{
	if (level == 2){
		kernelconf_probe(sim);
	}
	return LNCF_NOT_APPLICABLE;
}

static void usage()
{
	xconf_error (MSG_U(T_USAGE,"Module kernelconf\n"
		"linuxconf --modulemain kernelconf [ specific options ]\n"
		"\n"
		"    --set-file-max <value>\n"
		"    --default-file-max\n"
		"    --set-inode-max <value>\n"
		"    --default-inode-max\n"
		"    --set-dquot-max <value>\n"
		"    --default-dquot-max\n"
		"    --set-super-max <value>\n"
		"    --default-super-max\n"
		"    --enable-ctrl-alt-del\n"
		"    --disable-ctrl-alt-del\n"
		"    --default-ctrl-alt-del\n"
		"    --enable-fairscheduler\n"
		"    --disable-fairscheduler\n"
		"    --default-fairscheduler\n"
		"\n"
		));
}

PUBLIC void MODULE_kernelconf::usage (SSTRINGS &tb)
{
	tb.add (new SSTRING (MSG_R(T_USAGE)));
}

#define OPTION_ONOFF(x,o) \
	}else if (strcmp("--enable-" o,argv[i])==0){ \
		x##_set(1); \
		x##_save(1); \
	}else if (strcmp("--disable-" o,argv[i])==0){ \
		x##_set(0); \
		x##_save(0); \
	}else if (strcmp("--default-" o,argv[i])==0){ \
		x##_set(-1); 
#define OPTION_VAL(x,o) \
	}else if (strcmp("--set-" o,argv[i])==0){ \
		if (++i >= argc){ \
			::usage(); \
		}else{ \
			int val = atoi(argv[i]); \
			if (val <= 0){ \
				::usage(); \
			}else{ \
				x##_set(val); \
				x##_save(val); \
			} \
		} \
	}else if (strcmp("--default-" o,argv[i])==0){ \
		x##_set(-1);

PUBLIC int MODULE_kernelconf::execmain (int argc , char *argv[], bool)
{
	int ret = LNCF_NOT_APPLICABLE;
	const char *pt = strrchr(argv[0],'/');
	if (pt != NULL){
		pt++;
	}else{
		pt = argv[0];
	}
	if (strcmp(pt,"kernelconf")==0){
		ret = -1;
		if (argc == 1){
			while (1){
				if (kernelconf_edit() == -1) break;
				if (netconf_checkupdate(true)!=2) break;
			}
		}else{
			for (int i = 1; i < argc; i++){
				if (0){
				OPTION_VAL(file_max,"file-max")
				OPTION_VAL(inode_max,"inode-max")
				OPTION_VAL(dquot_max,"dquot-max")
				OPTION_VAL(super_max,"super-max")
				OPTION_ONOFF(fairscheduler,"fairscheduler")
				OPTION_ONOFF(ctrl_alt_del,"ctrl-alt-del")
				}else{
					::usage();
				}
			}
		}
	}
	return ret;
}


static MODULE_kernelconf kernelconf;

