
#include "kernelconf.h"
#include "kernelconf.m"
#include <netconf.h>

static HELP_FILE help_kernelconf("kernelconf","kernelconf");

#define ADD_CHK(x,p) \
	char x##val; \
	bool x##isavailable = x##_available(); \
	do { \
		if (x##isavailable) { \
			x##val = (x##_get()==-1)?2:!x##_get(); \
			dia.newf_chkm (p,x##val,chk); \
		} \
	} while (0)
#define GET_CHK(x) \
	do { \
		if (x##isavailable) { \
			if (x##val!=2){ \
				x##_set(!((int)x##val)); \
			}else{ \
				x##_set(-1); \
			} \
		} \
	} while (0)
#define ADD_NUM(x,p) \
	int x##val; \
	bool x##isavailable = x##_available(); \
	do { \
		if (x##isavailable) { \
			if (x##_get()==-1){ \
				x##val = -1; \
				dia.newf_chkm_num (p,x##val,x##_read(),num_vals,num); \
			} else { \
				x##val = x##_get(); \
				dia.newf_chkm_num (p,x##val,num_vals,num); \
			} \
		} \
	} while (0)
#define GET_NUM(x) \
	do { \
		if (x##isavailable) { \
			if (x##val!=-1){ \
				x##_set(x##val); \
			}else{ \
				x##_set(-1); \
			} \
		} \
	} while (0)

int kernelconf_edit()
{
	int ret = -1;
	const char *chk[] = {"enable","disable","default",NULL};
	const char *num[] = {"default",NULL};
	const int num_vals[] = {-1};
	DIALOG dia;
	dia.newf_title(MSG_U(F_FS,"File System"),1,"",MSG_R(F_FS));
	ADD_NUM(file_max, MSG_U(F_MAXFILE,"Max. file handles"));
	ADD_NUM(inode_max,MSG_U(F_MAXINODE,"Max. inode handles"));
	ADD_NUM(dquot_max,MSG_U(F_MAXDQUOT,"Max. num. cached disk quota"));
	ADD_NUM(super_max,MSG_U(F_MAXSUPER,"Max. super block handlers"));
	dia.newf_title(MSG_U(T_GENERAL,"General"),1,"",MSG_R(T_GENERAL));
	ADD_CHK(ctrl_alt_del,MSG_U(F_CTRLALTDEL,"Ctrl-alt-del"));
	ADD_CHK(fairscheduler,MSG_U(F_FAIRSCHED,"Fair scheduler"));
	int nof=0;
	while(1){
		MENU_STATUS code = dia.edit(
				MSG_U(T_KERNELCONF,"Kernel Configuration"),
				MSG_U(I_KERNELCONF, "This dialog allows you to change kernel options. BE CAREFUL!"),
				help_kernelconf,nof,
				MENUBUT_ACCEPT|MENUBUT_CANCEL);
		
		if (code == MENU_ESCAPE || code == MENU_QUIT || code == MENU_CANCEL){
			break;
		}else if (code == MENU_ACCEPT){
			GET_NUM(file_max);
			GET_NUM(inode_max);
			GET_NUM(dquot_max);
			GET_NUM(super_max);
			GET_CHK(ctrl_alt_del);
			GET_CHK(fairscheduler);
			ret = 0;
			break;
		}	
	}
	return ret;
}

#define PROBE_OPT(x) \
	do { \
		if (x##_available() && (x##_get() != -1) && (x##_read() != x##_get())) { \
			net_prtlog (NETLOG_CMD,"Executing: echo %d > %s\n", x##_get(), f_##x.getpath()); \
			if (!sim){ \
				x##_save(x##_get()); \
			} \
		} \
	} while (0)

int kernelconf_probe(bool sim)
{
	if (!sim){
		net_title(MSG_U(S_CHECKKERNEL,"Checking kernel configuration"));
	}
	PROBE_OPT(file_max);
	PROBE_OPT(inode_max);
	PROBE_OPT(dquot_max);
	PROBE_OPT(super_max);
	PROBE_OPT(ctrl_alt_del);
	PROBE_OPT(fairscheduler);
	return 0;
}

// vim: nowrap
