
#include "kernelconf.h"
#include <stdlib.h>

#define SUBSYS "kernel"

// Functions here follow the following naming rule:
//
// xxx_available() - tells if this functionality is available in the kernel
// xxx_read()      - return the current value used by the system
// xxx_save()      - sets the value used by the system
// xxx_get()       - return the configured value
// xxx_set()       - set this value as the configured value. If you pass
//                   -1 as parameter, it's going to unset any defined value

//
// Macros to define default functions
//

#define DEFINE_AVAILABLE(x) \
bool x##_available() \
{ \
	return f_##x.exist(); \
}

#define DEFINE_READ(x) \
int x##_read() \
{ \
	int ret = -1; \
	FILE_CFG *f = f_##x.fopen("r"); \
	char b[257]; \
	if (fgets(b,256,f) != NULL){ \
		ret = atoi(b); \
	} \
	fclose(f); \
	return ret; \
} 

#define DEFINE_SAVE(x) \
void x##_save(int val) \
{ \
	FILE_CFG *f = f_##x.fopen("w"); \
	fprintf(f,"%d",val); \
	fclose(f); \
}

#define DEFINE_GET(x,s) \
int x##_get() \
{ \
	linuxconf_setcursys(SUBSYS); \
	return linuxconf_getvalnum (#x,#s,-1); \
}

#define DEFINE_SET(x,s) \
void x##_set(int val) \
{ \
	linuxconf_setcursys(SUBSYS); \
	if (val != -1){ \
		if (x##_read() == -1){ \
			linuxconf_add (#x,#s,val); \
		}else{ \
			linuxconf_replace (#x,#s,val); \
		} \
	}else if (x##_read() != -1){ \
		linuxconf_removeall (#x,#s); \
	} \
	linuxconf_save(); \
}

//
// file_max functions
// 

DEFINE_AVAILABLE(file_max);
DEFINE_READ(file_max);
DEFINE_SAVE(file_max);
DEFINE_GET(file_max,value);
DEFINE_SET(file_max,value);

//
// inode_max functions
// 

DEFINE_AVAILABLE(inode_max);
DEFINE_READ(inode_max);
DEFINE_SAVE(inode_max);
DEFINE_GET(inode_max,value);
DEFINE_SET(inode_max,value);

//
// dquot_max functions
// 

DEFINE_AVAILABLE(dquot_max);
DEFINE_READ(dquot_max);
DEFINE_SAVE(dquot_max);
DEFINE_GET(dquot_max,value);
DEFINE_SET(dquot_max,value);

//
// super_max functions
// 

DEFINE_AVAILABLE(super_max);
DEFINE_READ(super_max);
DEFINE_SAVE(super_max);
DEFINE_GET(super_max,value);
DEFINE_SET(super_max,value);

//
// ctrl_alt_del functions
// 

DEFINE_AVAILABLE(ctrl_alt_del);
DEFINE_READ(ctrl_alt_del);
DEFINE_SAVE(ctrl_alt_del);
DEFINE_GET(ctrl_alt_del,enable);
DEFINE_SET(ctrl_alt_del,enable);

//
// fairscheduler functions
// 

DEFINE_AVAILABLE(fairscheduler);
DEFINE_READ(fairscheduler);
DEFINE_SAVE(fairscheduler);
DEFINE_GET(fairscheduler,enable);
DEFINE_SET(fairscheduler,enable);

// vim: nowrap
