/* the defines' table resides here, so the routines considered generic */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <popen.h>

/* needed for change_owner_and_group(...) */
#include <grp.h>
#include <pwd.h>

/* printer types... */
#define PRT_LOCAL 0
#define PRT_REMOTE 1
#define PRT_SMBWIN 2
#define PRT_NETWARE 3

/* maxmimum of printers to be defined */
#define MAX_PRINTERS 16

#define TYPICAL_STRING_SIZE 100
#define sb_size 100

/* defaults while creating new printers */
#define DEFAULT_PRINTERNAME "lp"
#define DEFAULT_PRTDEVICE "/dev/lp0"
#define DEFAULT_PRTREMOTEQUEUE "lp"
#define DEFAULT_PRTFULLPATHSMBFILTER "/usr/lib/rhs/rhs-printfilters/smbprint"
#define DEFAULT_PRTSMBDEVICE "/dev/null"
#define DEFAULT_PRTFULLPATHNCPFILTER "/usr/lib/rhs/rhs-printfilters/ncpprint"
#define DEFAULT_PRTNCPDEVICE "/dev/null"

#define PRTCP_FILE "/etc/printcap"

/* when creating a new filter */
#define FILTERSCRIPT_NAME "filter"

/* when creating a new printcap entry (note that there's some-kind-of support to other paths than /var/spool/lpd but not here!!! -- unlikely to be a problem in future though) */
#define DEFAULT_FILTERSCRIPT_NAME "/var/spool/lpd/%s/filter"

// #define MASTERFILTER_FULLPATH "/usr/lib/rhs/rhs-printfilters/master-filter"

/* this is used when creating a new printer and its files */
#define DEFAULT_SPOOLDIRS_HOME "/var/spool/lpd"

#define PRINTCAP_LOCATION "/etc/printcap"

// #define PRINTERDB_FILENAME "/usr/lib/rhs/rhs-printfilters/printerdb"

#define DOTCONFIG_FILENAME ".config"

/* this is used for defining lpd running automatically is it wasn't this way previously */
#define KLINKLEV2_FULLPATH "/etc/rc.d/rc2.d/K60lpd"
#define KLINKLEV3_FULLPATH "/etc/rc.d/rc3.d/K60lpd"
#define KLINKLEV4_FULLPATH "/etc/rc.d/rc4.d/K60lpd"
#define KLINKLEV5_FULLPATH "/etc/rc.d/rc5.d/K60lpd"
#define KLINKLEV6_FULLPATH "/etc/rc.d/rc6.d/K60lpd"
#define LPDSCRIPT_TO_LINK "/etc/rc.d/init.d/lpd"
#define SLINKLEV2_FULLPATH "/etc/rc.d/rc2.d/S60lpd"
#define SLINKLEV3_FULLPATH "/etc/rc.d/rc3.d/S60lpd"
#define SLINKLEV4_FULLPATH "/etc/rc.d/rc4.d/S60lpd"
#define SLINKLEV5_FULLPATH "/etc/rc.d/rc5.d/S60lpd"
#define SLINKLEV6_FULLPATH "/etc/rc.d/rc6.d/S60lpd"

#define LPRTYPE_OLD_LPR 1
#define LPRTYPE_LPRNG   2

/* END of defines' stuff */

int kind_of_lpr_system_installed;

struct t_dataprintcap{
        /* effective fields */
        char printer_name[sb_size];

        char af[sb_size];
        int  br;
        char cf[sb_size];
        char df[sb_size];
        int  fc;
        char ff[sb_size];
        int  fo;
        int  fs;
        char gf[sb_size];
        int  hl;
        int  ic;
        char if_[sb_size];
        char lf[sb_size];
        char lo[sb_size];
        char lp[sb_size];
        int  mx;
        char nd[sb_size];
        char nf[sb_size];
        char of[sb_size];
        int  pc;
        int  pl;
        int  pw;
        int  px;
        int  py;
        char rf[sb_size];
        char rg[sb_size];
        char rm[sb_size];
        char rp[sb_size];
        int  rs;
        int  rw;
        int  sb;
        int  sc;
        char sd[sb_size];
        int  sf;
        int  sh;
        char st[sb_size];
        char tf[sb_size];
        char tr[sb_size];
        char vf[sb_size];
};

t_dataprintcap my_printcap[MAX_PRINTERS];

struct t_general_printer_index{
    char devname[100]; /* lp, lp1, etc */
    int  exists;       /* if true, this data is valid */
    char type[100];    /* local, ncp, etc */
} general_printer_index[MAX_PRINTERS];

/* change owner and group of a given file/directory */
void change_owner_and_group(char *given_path, char *given_owner, char *given_group)
{
    gid_t         my_group_code=0;
    uid_t         my_user_code=0;
    struct group  *my_current_group;
    struct passwd *my_current_user;

    setgrent();
    while((my_current_group=getgrent())){
        if(!strcmp(my_current_group->gr_name, given_group)){
            my_group_code=my_current_group->gr_gid;
        }
    }
    endgrent();
    setpwent();
    while((my_current_user=getpwent())){
        if(!strcmp(my_current_user->pw_name, given_owner)){
            my_user_code=my_current_user->pw_uid;
        }
    }
    endpwent();

    chown(given_path, my_user_code, my_group_code);
}

/* protos */
int execute_proggy_and_get_stdout(char *given_command_line, char *output_buffer, int output_buffer_size);

/* analyses if given string has characters <32 or >127 or |*` and such things */
/* returns true is provided string is invalid (illegal characters present) */
/* if (corrected_version!=NULL), provides the same string but having the illegal characters substituted by _ */
int invalid_string_provided(char *given_string, char *corrected_version)
{
    char the_invalid_ones[]=" !@#$%^&*()=+/?\\|<>,[]{}~`'\0";
    char current_data;
    char *current_pointer;
    int  is_there_illegal_characters=0;

    if(corrected_version){
        strcpy(corrected_version, given_string);
        current_pointer=corrected_version;
    } else {
        current_pointer=given_string;
    }

    while((current_data=*current_pointer++)){
        if((current_data<32)|(current_data>126)){
            is_there_illegal_characters=1;
            if(corrected_version)
                *(current_pointer-1)='_';
        } else {
            if(strchr(the_invalid_ones, current_data)){
                is_there_illegal_characters=1;
                if(corrected_version)
                    *(current_pointer-1)='_';
            }
        }
    }

    return(is_there_illegal_characters);
}

/* dump given text to specified file, returns 0 if successful */
int dump_this_string_to_given_file(char *given_string, char *given_filename, char *given_path)
{
    char full_pathname[500];
    FILE *my_file;


    sprintf(full_pathname, "%s/%s", given_path, given_filename);

    if((my_file=fopen(full_pathname, "w"))){
        fputs(given_string, my_file);
        fclose(my_file);
        chmod(full_pathname, 511); /* -rwxrwxrwx */
        return(0);
    }
    return(1);
}

/* returns true is given filename exists */
int this_file_exists(char *which_file)
{
    FILE *my_file;

    if((my_file=fopen(which_file, "r"))){
        fclose(my_file);
        return(1);
    }
    return(0);
}

int just_executes_the_proggy(char *given_command_line)
{
    char my_command[500];
    char my_args[500];
    char *where_to_cut;

    my_command[0]=0;
    my_args[0]=0;
    strcpy(my_command, given_command_line);
    if((where_to_cut=strchr(my_command, ' '))){
        *where_to_cut=0;
        where_to_cut++;
        strcpy(my_args, where_to_cut);
    }

    return(netconf_system_if(my_command, my_args));
}

/* execute command line and gets the string output.
   returns the command returncode, or -1 if failure */
int execute_proggy_and_get_stdout(char *given_command_line, char *output_buffer, int output_buffer_size)
{

/*    FILE *my_pseudo_file;
    int  myretcod;
    char mytrash[1000];
    int  my_space_left;
    char *where_to_write;

    my_space_left=output_buffer_size;
    if(my_space_left)     // for safety purposes
        my_space_left--;
    where_to_write=output_buffer;
    myretcod=-1;
    if(output_buffer)
        *output_buffer=0;

    if((my_pseudo_file=popen(given_command_line, "r"))){
        while(my_space_left){
            if(!fgets(where_to_write, my_space_left, my_pseudo_file)){
                my_space_left=0;
            } else {
                my_space_left-=strlen(where_to_write);
                where_to_write+=strlen(where_to_write);
            }
        }
        while(fgets(mytrash, 800, my_pseudo_file));
        myretcod=pclose(my_pseudo_file);
    }
    return(myretcod);
    */

    char my_command[500];
    char my_args[500];
    char *where_to_cut;

    *output_buffer=0;

    my_command[0]=0;
    my_args[0]=0;
    strcpy(my_command, given_command_line);
    if((where_to_cut=strchr(my_command, ' '))){
        *where_to_cut=0;
        where_to_cut++;
        strcpy(my_args, where_to_cut);
    }

    POPEN my_popen(my_command, my_args);
    while(!my_popen.wait(5000, 0));

    {
        int my_rem_bytes=output_buffer_size;
        int my_step;

        while((my_step=my_popen.readoutraw((output_buffer+strlen(output_buffer)), my_rem_bytes))){
            my_rem_bytes-=my_step;
            my_popen.wait(5000, 0);
        }
    }

    return(my_popen.close());
}

/* erase all directory's files and, then, the directory itself */
/* (does not remove internal subdirectories) */
/* returns zero if ok, nonzero if unsuccessful */
int zaapboom_directory(char *which_one)
{
    DIR *my_dir;
    struct dirent *ent;
    char tempy[2048];

    if((my_dir=opendir(which_one))){
        while((ent=readdir(my_dir))){
            if(strcmp(ent->d_name, ".")&&strcmp(ent->d_name, "..")){
                sprintf(tempy, "%s/%s", which_one, ent->d_name);
                unlink(tempy);
            }
        }
        closedir(my_dir);
    }
    return(rmdir(which_one));
}

void informational_window(const char *my_title, const char *my_text)
{
        int nothing=0;
        DIALOG uhoh_dialog;

        uhoh_dialog.editmenu(my_title, my_text, help_nil, nothing, 0);
}

/* loads the pointed .config (if it exists) file and returns the printer type
0- local or remote
PRT_SMBWIN 2
PRT_NETWARE 3
 used because the return_printer_type(N) is not absolutely
 correct while determining SMB or NCP printers */
int return_printer_type_from_generalcfg(char *given_filename)
{
    FILE *dotconfigfile;
    int  myfilesize;
    char *my_buff;
    int  my_prt_type=0;

    if((dotconfigfile=fopen(given_filename, "r"))){
        fseek(dotconfigfile, 0, SEEK_END);
        myfilesize=ftell(dotconfigfile);
        fseek(dotconfigfile, 0, SEEK_SET);

        if((my_buff=(char *)malloc(myfilesize+1))){
            if(fread(my_buff, myfilesize, 1, dotconfigfile)){
                *(my_buff+myfilesize)=0;
                if(strstr(my_buff, "share=")||
                   strstr(my_buff, "hostip=")||
                   strstr(my_buff, "workgroup="))
                    my_prt_type=PRT_SMBWIN; /* smb */
                if(strstr(my_buff, "server=")||
                   strstr(my_buff, "queue="))
                    my_prt_type=PRT_NETWARE; /* ncp */
            }
            free(my_buff);
        }
        fclose(dotconfigfile);
    }
    return(my_prt_type);
}

/* looks at variables and returns the printer type analysing the context */
/* from printcap file */
int return_printer_type(int which_printer)
{
    if(strlen(my_printcap[which_printer].rm))
        return(PRT_REMOTE);


    /* determines if printer is SMB or NCP */
    if(!strcmp(my_printcap[which_printer].lp, "/dev/null")){
        char dotconfig_fullpath[500];
        int my_prt_type;

        sprintf(dotconfig_fullpath, "%s/%s", my_printcap[which_printer].sd, DOTCONFIG_FILENAME);

        /* .config file analysis result takes priority over printcap's */
        if((my_prt_type=return_printer_type_from_generalcfg(dotconfig_fullpath))){
            return(my_prt_type);
        } else {
            if(strstr(my_printcap[which_printer].if_, "smb"))
                return(PRT_SMBWIN);
            return(PRT_NETWARE);
        }
    }
    return(PRT_LOCAL);
}

/* return true if yes, false if no */
int confirm_yesno_window(const char *my_title, const char *my_text)
{
    if(dialog_yesno(my_title, my_text, help_nil)==MENU_YES){
        return(1);
    } else {
        return(0);
    }
}

