/*
 this part manages:
 the main window (selects printer edit, allowed hosts and spool manager)
    (also manages printcap, but the heavy part of printcap managing is in another listing)
 the printerlist edit and printer add option

 it also have the routines for activating the lpd for startup for
 runlevels 2, 3, 4 and 5 if it isn't starting automatically and
 user confirms that it should be starting instead
 */
/*

TO DO..

- better perm_access() support (permmiting non-root users to manage the queue)
- sub systems definition (for rhs-printfilters)

*/


#include <stdlib.h>
#include <string.h>
#include "printer.h"
#include "printer.m"

#include <unistd.h>
#include <dirent.h>

#include <grp.h>
#include <pwd.h>
#include <sys/types.h>

#include "deez_parser_handle.h"
#include "printer_printcap.h"
#include "printer_props.cc"
#include "printer_common_data.h"
#include "printer_netwauth.cc"
#include "printer_cfile.h"
#include "printer_queue.cc"

#include <dialog.h>
#include <translat.h>


// static HELP_FILE my_main_helpfile(NULL, "apache");


/* returns true if printcap is LPRng-style (no reversed dashes present).. */
/* it'll also return false if file cannot be readen.. well.. */
int is_printcap_lprng_style(void)
{
    FILE_CFG *printcapfile;
    int  myfilesize;
    char *my_buff;
    int  what_code_to_return=0;

    if((printcapfile=cf_printcapfile.fopen("r"))){
    fseek(printcapfile, 0, SEEK_END);
        myfilesize=ftell(printcapfile);
        fseek(printcapfile, 0, SEEK_SET);

        if((my_buff=(char *)malloc(myfilesize+1))){
            *my_buff=0;
            *(my_buff+myfilesize)=0;
            fread(my_buff, myfilesize, 1, printcapfile);

            /* here it analyses the file data */
            if(strchr(my_buff, ':')){
                char *my_pointer;

                what_code_to_return=1;
                my_pointer=my_buff;
                while((my_pointer=strchr(my_pointer, '\\'))){
                    my_pointer++;
                    while(*my_pointer==' ')
                        my_pointer++;
                    if(*my_pointer=='\n')
                        what_code_to_return=0;
                }
            }

            free(my_buff);
        }
        cf_printcapfile.fclose(printcapfile);
    }
    return(what_code_to_return);
}

/* checks if lpd is starting automatically, if yes do nothing.
 if not, ask user if (s)he wants to change it and, if confirmed,
 - makes lpd starting automatically
 - starts lpd */
void check_lpd_status_and_ask_user(void)
{
    char my_work_string[500];
    char my_fullpath_string[500];
    int  current_runlevel=0;

    /* verifies if lpd is starting automatically for the current runlevel */
    execute_proggy_and_get_stdout("runlevel", my_work_string, 500);
    if(parse_integer_data(my_work_string, "N ", &current_runlevel)){
        if(!current_runlevel)
            current_runlevel=3;
        sprintf(my_fullpath_string, "/etc/rc.d/rc%d.d/S60lpd", current_runlevel);
        if(!this_file_exists(my_fullpath_string)){
            sprintf(my_work_string, MSG_U(M_LPDNOTSTTX, "\"lpd\" is not being started automatically for the current runlevel (%d).\nIn order to be able to print you need it running.\n\nDo you want \"lpd\" always starting automatically (for runlevels 2, 3, 4 and 5)?\n(When Linuxconf asks if you want changes taking effect or not, reply positively)"), current_runlevel);

            if(confirm_yesno_window(MSG_U(M_LPDNOTSTTT, "warning about lpd startup"), my_work_string)){
                /* set lpd starting for runlevels 2, 3, 4 and 5.. stopping at 6 */
                unlink(KLINKLEV2_FULLPATH);
                unlink(KLINKLEV3_FULLPATH);
                unlink(KLINKLEV4_FULLPATH);
                unlink(KLINKLEV5_FULLPATH);
                unlink(SLINKLEV6_FULLPATH);

                symlink(LPDSCRIPT_TO_LINK, SLINKLEV2_FULLPATH);
                symlink(LPDSCRIPT_TO_LINK, SLINKLEV3_FULLPATH);
                symlink(LPDSCRIPT_TO_LINK, SLINKLEV4_FULLPATH);
                symlink(LPDSCRIPT_TO_LINK, SLINKLEV5_FULLPATH);
                symlink(LPDSCRIPT_TO_LINK, KLINKLEV6_FULLPATH);
            }
        }
    }
}

/* create directory and all initially needed stuff */
/* non-zero = failure */
int create_printer_files(char *printer_name_full_path)
{
    char tempy[100];

    strcpy(tempy, printer_name_full_path);
    if(mkdir(tempy, 0)){


        // ADICIONAR ROTINA DE VERIFICAO SE  REALMENTE UM DIRETRIO
        if(!confirm_yesno_window(MSG_U(M_COULDNTCREATEPRTDIRTT, "Unable to create printer directory"), MSG_U(M_COULDNTCREATEPRTDIRTX2, "It's likely there's already a spool directory\nwith the same name as this printer\nyou want to create.\n\nPerhaps this directory was left when you\ndeleted the printer which used it.\n\nDo you want to use this directory for the new printer?\n(its contents will be left untouched)"))){
            return(1);
        }
//        informational_window(MxxSxxG_U(M_COULDNTCREATEPRTDIRTT, "Unable to create printer directory"),
//                             MxxSxxG_U(M_COULDNTCREATEPRTDIRTX, "An error occurred while creating the printer directory.\nTry using another printer name."));

    }
    else
    {
        chmod(tempy, 493); /* -rwxr-xr-x */

        /* change directory group and owner to 'root/lp', 'daemon/daemon'..
           according the lpr system installed */
        if(kind_of_lpr_system_installed==LPRTYPE_LPRNG){
            change_owner_and_group(tempy, "daemon", "daemon");
        } else {
            change_owner_and_group(tempy, "root", "lp");
        }
    }

    return(0);
}

/* load printcap data (other printers' data will be loaded in different parts of code) */
void load_printcap(void)
{
    int i;

    i=MAX_PRINTERS;
    while(i--){
        general_printer_index[i].exists=0;
        *(general_printer_index[i].devname)=0;
        *(general_printer_index[i].type)=0;
    }

    i=0;
    while((load_dataprintcap(&my_printcap[i], i))&&(i<MAX_PRINTERS)){
        general_printer_index[i].exists=1;
        strcpy(general_printer_index[i].devname, my_printcap[i].printer_name);
        i++;
    }
}

void save_all_printers_data(void)
{
    FILE_CFG *printcap_file;
    int  i;
       
//    if((printcap_file=fopen(PRINTCAP_LOCATION, "w"))){
    if((printcap_file=cf_printcapfile.fopen("w"))){
    char tempy[300];
        
        sprintf(tempy, "# %s\n#\n# This file was generated by Linuxconf's Printer module\n", PRINTCAP_LOCATION);
        fputs(tempy, printcap_file);
        sprintf(tempy, "# You may edit this file directly, but only standard printcap entries\n# will be preserved.\n");
        fputs(tempy, printcap_file);

        i=0;
        while (i<MAX_PRINTERS){
            if(general_printer_index[i].exists){
                /* insert ##PRINTTOOL3## line */
                load_initialise_secondary_configfiles(i);
                create_commented_printcap_config(tempy);
                fputs("\n", printcap_file);
                fputs(tempy, printcap_file);
                fputs("\n", printcap_file);
                /* now inserts the conventional printcap data */
                sprintf(tempy, "%s:", general_printer_index[i].devname);
                save_dataprintcap(&my_printcap[i], printcap_file, tempy);
            }
            i++;
        }
        
        sprintf(tempy, "\n");
        fputs(tempy, printcap_file);
//        fclose(printcap_file);
        cf_printcapfile.fclose(printcap_file);
    }
}

/* returns true if printer was successfully added */
int addprinter_window(void)
{
    int     i;
    int     free_slot;
    int     printername_already_taken;
    char    printer_name[30];
    char    type_printr;
    int     my_returncode;
    char    my_pathspooldirectories[TYPICAL_STRING_SIZE];
    char    my_fullspoolpath[500];

    strcpy(my_pathspooldirectories, DEFAULT_SPOOLDIRS_HOME);
    my_returncode=0; /* useless, present only for compiler warning supressing */

    free_slot=-1;
    i=0;
    while(i<MAX_PRINTERS){
        if(general_printer_index[i].exists){
            i++;
        } else {
            free_slot=i;
            i=MAX_PRINTERS;
        }
    }
    if(free_slot==-1){
        informational_window(MSG_U(M_MAXPRINTERSREACHEDTT, "Maximum printers' limit reached"), MSG_U(M_MAXPRINTERSREACHEDTX, "Too many printers attached to this system.\nRemove some printer and try again."));
        return(0);
    }

    {
        DIALOG addprinter_dialog;

        *printer_name=0;
        type_printr=0;

        /* suggests an unique queuename */
        {
            char proposal_of_printername[TYPICAL_STRING_SIZE];
            int i=0;
            int  proposal_number=1;

            while(i<MAX_PRINTERS){
                if(proposal_number>1){
                    sprintf(proposal_of_printername, "%s%d", DEFAULT_PRINTERNAME, proposal_number);
                } else {
                    strcpy(proposal_of_printername, DEFAULT_PRINTERNAME);
                }
                if(!strcmp(my_printcap[i].printer_name, proposal_of_printername)){
                    i=0;
                    proposal_number++;
                } else {
                    i++;
                }
            }
            strcpy(printer_name, proposal_of_printername);  /* suggested printername given to the user */
        }

        addprinter_dialog.newf_str(MSG_U(M_NEWPRINTERNAME, "Printer name"), printer_name, 20);

        /* uncomment this line if you want to provide an alternative path for spool directories (so the new spool will be created there) */
        //        addprinter_dialog.newf_str("Path for spool diretories:", my_pathspooldirectories, TYPICAL_STRING_SIZE-1);

        addprinter_dialog.newf_title("", MSG_U(M_NEWPRTATTACHEDAS, "Attached as"));
        addprinter_dialog.newf_radio("", type_printr, PRT_LOCAL, MSG_U(M_NEWPRTLOCAL, "Local Printer"));
        addprinter_dialog.newf_radio("", type_printr, PRT_REMOTE, MSG_U(M_NEWPRTREMOTE, "Remote Queue"));
        addprinter_dialog.newf_radio("", type_printr, PRT_SMBWIN, MSG_U(M_NEWPRTSMBWIN, "SMB/Windows Printer"));
        addprinter_dialog.newf_radio("", type_printr, PRT_NETWARE, MSG_U(M_NEWPRTNETW, "NetWare Printer (NCP)"));

        my_returncode=addprinter_dialog.edit(MSG_U(M_ADDINGPRINTERTT, "Adding a printer..."), MSG_U(M_ADDINGPRINTERTX, "Please specify a name and how the printer is connected.\n\"lp\" is the default system printer.\n"), help_nil);
        sprintf(my_fullspoolpath, "%s/%s", my_pathspooldirectories, printer_name);
    }

    if((my_returncode==MENU_CANCEL)||(my_returncode==MENU_ESCAPE))
        return(0);

    if(!printer_name[0]){
        informational_window(MSG_U(M_PRTNAMEUNDEFINEDTT, "Printer name not defined"),
                             MSG_U(M_PRTNAMEUNDEFINEDTX, "You must define a name for each added printer"));
        return(0);
    }

    if(invalid_string_provided(printer_name, NULL)){
        informational_window(MSG_U(M_PRTNAMEINVALIDTT, "Invalid printer name provided"),
                             MSG_U(M_PRTNAMEINVALIDTX, "There are certain characters which can't be used while naming the printer.\n(space and control characters are not supported, try using Aa-Zz 0-9)"));
        return(0);
    }

    i=MAX_PRINTERS;
    printername_already_taken=0;
    while(i--){
        if((general_printer_index[i].exists)&&(!strcmp(printer_name, general_printer_index[i].devname)))
            printername_already_taken=1;
    }
    if(printername_already_taken){
        informational_window(MSG_U(M_PRTNAMEREPEATEDTT, "Repeated printer name given"),
                             MSG_U(M_PRTNAMEREPEATEDTX, "You must define an unique name for each printer"));
        return(0);
    }
    
    if(create_printer_files(my_fullspoolpath))
        return(0);
    
    /* setup new printer defaults */
    general_printer_index[free_slot].exists=1;
    strcpy(general_printer_index[free_slot].devname, printer_name);
    load_printcap_defaults(&my_printcap[free_slot]);
    strcpy(my_printcap[free_slot].sd, my_fullspoolpath);
    sprintf(my_printcap[free_slot].if_, DEFAULT_FILTERSCRIPT_NAME, printer_name);

    /* now the printer is added to system */
    switch(type_printr)
    {
    case PRT_LOCAL:
        strcpy(my_printcap[free_slot].lp, DEFAULT_PRTDEVICE);
        break;
    case PRT_REMOTE:
        strcpy(my_printcap[free_slot].rp, DEFAULT_PRTREMOTEQUEUE);
        strcpy(my_printcap[free_slot].rm, MSG_U(M_SOMEMACHMYNETWORK, "somemachine.mynetwork"));
        break;
    case PRT_SMBWIN:
        strcpy(my_printcap[free_slot].if_, DEFAULT_PRTFULLPATHSMBFILTER);
        strcpy(my_printcap[free_slot].lp, DEFAULT_PRTSMBDEVICE);
        dump_this_string_to_given_file("share=\'\\\\my_hostname\\my_printername\'\nhostip=\nuser=\'username_for_this\'\npassword=\'\'\nworkgroup=\'my_workgroup\'\n",
                                       DOTCONFIG_FILENAME,
                                       my_printcap[free_slot].sd);
        break;
    case PRT_NETWARE:
        strcpy(my_printcap[free_slot].if_, DEFAULT_PRTFULLPATHNCPFILTER);
        strcpy(my_printcap[free_slot].lp, DEFAULT_PRTNCPDEVICE);
        dump_this_string_to_given_file("server=my_server\nqueue=my_queue\nuser=username_for_this\npassword=\n",
                                       DOTCONFIG_FILENAME,
                                       my_printcap[free_slot].sd);
        // sprintf(my_printcap[free_slot].af, DEFAULT_PRTACCOUNT_NAME, printer_name);
        break;
    }


    save_all_printers_data();

    return(1);
}

void manage_printerslistwindow(void)
{
    int my_selection;
    MENU_STATUS my_button;
    DIALOG_RECORDS main_dialog;
    int total_items_added;

    my_selection=0;

    while(1){
        {
//            DIALOG_RECORDS main_dialog;
            main_dialog.remove_all();
            int i;

            load_printcap();

            main_dialog.newf_head("", MSG_U(M_PRTNAMEDESCTABLE, "printer name \tdescription"));

            total_items_added=0;
            i=0;
            while(i<MAX_PRINTERS){
                if(general_printer_index[i].exists){
                    char tempz[100];

                    switch(return_printer_type(i)){
                    case PRT_LOCAL:
                        strcpy(tempz, MSG_U(M_PTYPETABLE_LOCAL, "local"));
                        break;
                    case PRT_REMOTE:
                        strcpy(tempz, MSG_U(M_PTYPETABLE_REMOTE, "remote"));
                        break;
                    case PRT_SMBWIN:
                        strcpy(tempz, MSG_U(M_PTYPETABLE_SMBWIN, "SMB/Windows"));
                        break;
                    case PRT_NETWARE:
                        strcpy(tempz, MSG_U(M_PTYPETABLE_NETW, "Netware"));
                        break;
                    }
                    strcat(tempz, " \t");
                    main_dialog.new_menuitem(general_printer_index[i].devname, tempz);
                    total_items_added++;
                }
                i++;
            }

            my_button=main_dialog.editmenu(MSG_U(M_PRINTERSSETUP, "Printers' Setup"), MSG_U(M_CLICKTHEPRINTERNAME, "Click on the printername\nto edit its properties.\n"), my_edit_helpfile, my_selection, MENUBUT_QUIT|MENUBUT_ADD);
        }

        switch(my_button){
        case MENU_ADD:
            if(addprinter_window()){
                informational_window(MSG_U(M_PRTJUSTCREATEDTT, "Printer successfully created"), MSG_U(M_PRTJUSTCREATEDTX, "The name of the printer you just created\nwill appear in the printerlist.\n\nClick on its name in order to edit its properties."));
            }
            break;
        case MENU_QUIT:
        case MENU_ESCAPE:
            return;
        default:
            /* edit specified printer */
            if(total_items_added){
                if(editprinter_window(my_selection))
                    save_all_printers_data();
            }
            break;
        }
    }
}

/* shows and manage the main menu */
/* when it returns the module is supposed to quit */
void manage_mainmenu()
{
    MENU_STATUS my_button;
    int    my_selection;
    DIALOG_MENU dia_mymenu;

    my_selection=0;
    dia_mymenu.new_menuitem("", "", MSG_U(M_MAINADDEDPRINTERS, "Add/Edit printers"));
    dia_mymenu.new_menuitem("", "", MSG_U(M_MAINNETWKAUTH, "Network authorizations"));
    dia_mymenu.new_menuitem("", "", MSG_U(M_MAINQUEUEMGR, "Queue manager"));

    while(1){
        my_button=dia_mymenu.editmenu(MSG_U(M_MAINDIALOGTT, "Printer services setup"), "", my_main_helpfile, my_selection, MENUBUT_QUIT);
        if((my_button==MENU_QUIT)||(my_button==MENU_ESCAPE))
            return;

        switch(my_selection){
        case 0:
            manage_printerslistwindow();
            break;
        case 1:
            edit_whocanuse_myprinters();
            break;
        case 2:
            queue_main_window();
            break;
        }
    }
}

/* says if old lpr system is being used, or LPRng.. */
/* (currently only old_RH_lpr and LPRng are supported) */
int return_lpr_system_installed_in_this_system(void)
{
    char fried_chicken[1024];

    execute_proggy_and_get_stdout("lpd -V 2>&1", fried_chicken, 1023);
    if(strstr(fried_chicken, "LPRng")){
        return(LPRTYPE_LPRNG);
    }

    return(LPRTYPE_OLD_LPR);
}

void printer_ed (void)
{
    if(perm_rootaccess("")){

        if(is_printcap_lprng_style()){
            /* printcap IS LPRng styled, unsupported */
            informational_window(MSG_U(M_LPRNGPRINTCAPTT, "LPRng-style printcap file found"),
                                 MSG_U(M_LPRNGPRINTCAPTX, "This module can't manage this printcap variant.\nYou may convert the file to the classic\nprintcap format or delete it and\nlet this module recreate a fresh-new\nprintcap file.\n"));
        } else {
            /* printcap is the classic lpr format, so it's ok */

            /* defines that global variable with the type of lpr system installed on machine */
            kind_of_lpr_system_installed=return_lpr_system_installed_in_this_system();

            /* verify if lpd is being automatically initialised */
            check_lpd_status_and_ask_user();

            /* main menu */
            manage_mainmenu();

            /* at this point the module concluded operation... */
        }
    }
}
