#include <dialog.h>
#include <diajava.h>
#include "tlmpdia.h"
#include "tlmpdia.m"

void _F_editmenu::exec(const char *)
{
}
void _F_editmenu::execthread(const char *)
{
}

struct EDITMENU_PRIVATE {
	SSTRINGS actives;		// Which menu entries are already active
	const char *key;
	PRIVATE_MESSAGE msg;
};


static void ft(void *p)
{
	_F_editmenu *c = (_F_editmenu*)p;
	EDITMENU_PRIVATE *priv = c->priv;
	const char *key = priv->key;
	// Make sure we are not reentering the same option twice
	if (priv->actives.lookup(key)==-1){
		SSTRING *s = new SSTRING(key);
		priv->actives.add (s);
		c->execthread (key);
		priv->actives.remove_del(s);
		dialog_sendmessage (priv->msg);
	}
}

void editmenu (
	_F_editmenu &c,
	const char *title,
	const char *intro,
	HELP_FILE &help,
	const char *menuopt[])
{
	EDITMENU_PRIVATE priv;
	c.priv = &priv;
	DIALOG_MENU dia;
	dia.new_menuitems (menuopt);
	dia.waitfor (priv.msg);
	int nof = 0;
	if (title == NULL) title = MSG_U(T_NOTITLE,"Untitled");
	bool hidden = false;
	int nbthread = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (title,intro,help,nof,0);
		if (code == MENU_QUIT || code == MENU_ESCAPE){
			if (nbthread > 0){
				dia.hide();
				hidden = true;
			}else{
				break;
			}
		}else if (code == MENU_MESSAGE){
			nbthread--;
			if (nbthread == 0 && hidden) break;
		}else{
			priv.key = dia.getmenustr(nof);
			c.exec (priv.key);
			if (dialog_mode == DIALOG_GUI && 0){
				uithread (ft,(void*)&c);
				nbthread++;
			}else{
				c.execthread (priv.key);
			}
		}
	}
}


enum DIALOG_LOAD { LOAD_REAL, LOAD_TEST};

struct EDITRECORDS_PRIVATE {
	DIALOG_LOAD status;
	struct {
		SSTRING head;			// header of the dialog
		SSTRING_KEYS records;	// Current content of the dialog
	} current, accum;
	DIALOG_RECORDS *dia;
	int buttons;
	int nbitems;
	bool loadedonce;		// We have executed head() and load() once
	bool mustexit;			// We must terminate edition
	int nof;				// Current item
};

/*
	Generally called from editone to request editrecords to teminate
*/
void _F_editrecords::endedit()
{
	priv->mustexit = true;
}

/*
	Set the current field
*/
void _F_editrecords::setcursor(int no)
{
	priv->nof = no;
}

void _F_editrecords::new_menuitem (const char *s1, const char *s2)
{
	if (priv->status == LOAD_REAL){
		priv->dia->set_menuitem (priv->nbitems++,s1,s2);
		priv->current.records.add (s1,s2);
	}else{
		priv->accum.records.add (s1,s2);
	}
}

void _F_editrecords::newf_head (const char *s)
{
	if (priv->status == LOAD_REAL){
		priv->dia->newf_head ("",s);
		priv->current.head.setfrom (s);
	}else{
		priv->accum.head.setfrom (s);
	}
}

void _F_editrecords::addwhat(const char *s)
{
	if (priv->status == LOAD_REAL){
		priv->dia->addwhat (s);
		priv->buttons |= MENUBUT_ADD;
	}
}
void _F_editrecords::delwhat(const char *s)
{
	if (priv->status == LOAD_REAL){
		priv->dia->delwhat (s);
		priv->buttons |= MENUBUT_DEL;
	}
}
void _F_editrecords::setbutinfo (int id,
	 const char *title,
	 const char *icon)
{
	if (!priv->loadedonce){
		priv->dia->setbutinfo (id,title,icon);
		int butmask = MENUBUT_USR1 << (id - MENU_USR1);
		priv->buttons |= butmask;
	}
}

void _F_editrecords::add ()
{
}

void _F_editrecords::otherbuttons (MENU_STATUS)
{
}
void _F_editrecords::message ()
{
}

void _F_editrecords::setcontext (const char *s)
{
	priv->dia->setcontext (s);
}
void _F_editrecords::set_button_on_side ()
{
	priv->dia->set_button_on_side ();
}
void _F_editrecords::waitfor (PRIVATE_MESSAGE &msg)
{
	priv->dia->waitfor (msg);
}

void _F_editrecords::settype (DIALOG_TYPE type)
{
	priv->dia->settype (type);
}

void _F_editrecords::editone (int no, UISTATE &state)
{
	pickone (no,state);
	endedit();
}

void _F_editrecords::pickone (int , UISTATE &)
{
	xconf_notice (MSG_U(N_NOPICKONE,"Function/tag pickone or editone must be defined"));
}



static void editcommon (
	DIALOG_RECORDS &dia,
	_F_editrecords &c,
	const char *title,
	const char *intro,
	HELP_FILE &help,
	const char *diactx)
{
	EDITRECORDS_PRIVATE priv;
	dia.setcontext (diactx);
	c.priv = &priv;
	priv.nof = 0;
	priv.dia = &dia;
	priv.status = LOAD_REAL;
	priv.loadedonce = false;
	priv.buttons = 0;
	priv.mustexit = false;
	c.head ();
	int nbhead = dia.getnb();
	while (1){
		if (priv.mustexit) break;
		priv.nbitems = 0;
		priv.current.head.setfrom ("");
		priv.current.records.remove_all ();
		c.load ();
		priv.loadedonce = true;
		while (dia.getnb() > priv.nbitems + nbhead){
			dia.remove_del(priv.nbitems+nbhead);
		}
		MENU_STATUS code = dia.editmenu (title,intro,help
			,priv.nof,priv.buttons);
		if (code == MENU_QUIT || code == MENU_ESCAPE){
			break;
		}else if (code == MENU_ADD){
			c.add();
		}else if (code == MENU_OK){
			if (priv.nof >= 0 && priv.nof < priv.nbitems){
				UISTATE state;
				diajava_lastmousestate (state);
				c.editone (priv.nof,state);
			}
		}else if (code == MENU_MESSAGE){
			c.message();
		}else{
			c.otherbuttons(code);
		}
		#if 0
			priv.status = LOAD_TEST;
			priv.accum.head.setfrom ("");
			priv.accum.records.remove_all ();
			c.head ();
			c.load ();
			if (priv.accum.head.cmp(priv.current.head)!=0
				|| priv.accum.records.getnb() != priv.current.records.getnb()){
				dia.remove_all();
			}else{
				for (int i=0; i<priv.accum.records.getnb(); i++){
					SSTRING_KEY *n = priv.accum.records.getitem(i);
					SSTRING_KEY *o = priv.current.records.getitem(i);
					if (o->cmp(*n)!=0 ||
						o->getobj()->cmp(n->getobjval())!=0){
						dia.set_menuitem (i,n->get(),n->getobjval());
						o->setfrom(*n);
						o->getobj()->setfrom (n->getobjval());
					}
				}
			}
		#endif
	}
}

void editmanyrecords (
	_F_editmanyrecords &c,
	const char *title,
	const char *intro,
	HELP_FILE &help,
	const char *diactx)
{
	DIALOG_LISTE dia;
	editcommon (dia,c,title,intro,help,diactx);
}

void editmanyrecords (
	_F_editmanyrecords &c,
	const char *title,
	const char *intro,
	HELP_FILE &help)
{
	editmanyrecords (c,title,intro,help,NULL);
}

void editrecords (
	_F_editrecords &c,
	const char *title,
	const char *intro,
	HELP_FILE &help,
	const char *diactx)
{
	DIALOG_RECORDS dia;
	editcommon (dia,c,title,intro,help,diactx);
}

void editrecords (
	_F_editrecords &c,
	const char *title,
	const char *intro,
	HELP_FILE &help)
{
	editrecords (c,title,intro,help,NULL);
}

