#include <sys/types.h>
#include <dirent.h>
#include <popen.h>
#include "tlmplib.h"

bool _F_walkfs::recurse(const char *)
{
	return true;
}


int walkfs (_F_walkfs &c, const char *dirpath)
{
	int ret = -1;
	DIR *dir = opendir (dirpath);
	if (dir != NULL){
		ret = 0;
		struct dirent *ent;
		while ((ent = readdir (dir)) != NULL){
			if (strcmp(ent->d_name,".")!=0
				&& strcmp(ent->d_name,"..")!=0){
				char path[PATH_MAX];
				snprintf (path,sizeof(path)-1,"%s/%s",dirpath,ent->d_name);
				c.onefile (path);
				if (file_type(path)==1
					&& c.recurse(path)){
					// This is a directory and we are allowed to recurse
					ret += walkfs (c,path);
				}
				ret ++;
			}
		}
		closedir (dir);
	}
	return ret;
}

int _F_walkpopen::oneerr(const char *, const char *)
{
	return 0;
}
void _F_walkpopen::start()
{
}
void _F_walkpopen::end()
{
}
void _F_walkpopen::empty()
{
}

static int walkpopen_common (
	_F_walkpopen &c,
	POPEN &pop,
	int timeout,
	const char *command)
{
	int ret = -1;
	if (pop.isok()){
		ret = 0;
		bool start_called = false;
		while (1){
			if (pop.wait(timeout)< 0) break;
			char buf[PATH_MAX];
			while (pop.readout(buf,sizeof(buf))!=-1){
				int last = strlen(buf) - 1;
				if (last >= 0 && buf[last] == '\n') buf[last] = '\0';
				if (ret == 0){
					c.start();
					start_called = true;
				}
				if (c.oneline (buf,ret)==-1){
					ret = -1;
					break;
				}
				ret ++;
			}
			while (pop.readerr(buf,sizeof(buf))!=-1){
				int last = strlen(buf) - 1;
				if (last >= 0 && buf[last] == '\n') buf[last] = '\0';
				if (c.oneerr (buf,command)==-1){
					ret = -1;
					break;
				}
			}
		}
		if (ret == 0){
			c.empty();
		}else if (start_called){
			c.end();
		}
	}
	return ret;

}

int walkpopen (
	_F_walkpopen &c,
	const char *command,
	int timeout)
{
	POPEN pop (command);
	return walkpopen_common (c,pop,timeout,command);
}
int walkpopen (
	_F_walkpopen &c,
	const char *command,
	const char *arg,
	int timeout)
{
	POPEN pop (command,arg);
	SSTRING full;
	full.setfromf ("%s %s",command,arg);
	return walkpopen_common (c,pop,timeout,full.get());
}

